@@ -2,9 +2,7 @@ import re
 import sys
 import argparse
 import os
-import json
 import time
-import shutil
 
 from bilili.utils.base import repair_filename, touch_dir, touch_file, size_format
 from bilili.utils.playlist import Dpl, M3u
@@ -13,34 +11,40 @@ from bilili.utils.console import (Console, Font, Line, String, ProgressBar,
                                   List, DynamicSymbol, ColorString)
 from bilili.utils.subtitle import Subtitle
 from bilili.utils.attrdict import AttrDict
+from bilili.tools import spider, ass, regex
+from bilili.tools import global_status
+from bilili.handlers.downloader import RemoteFile
+from bilili.handlers.merger import MergingFile
+from bilili.video import BililiContainer
 from bilili.api.subtitle import get_subtitle
 from bilili.api.danmaku import get_danmaku
-from bilili.tools import spider, ass, regex
-from bilili.tools import global_middleware
-from bilili.events.downloader import RemoteFile
-from bilili.events.merger import MergingFile
-from bilili.video import BililiContainer
 from bilili.api.exceptions import (ArgumentsError, CannotDownloadError,
                                    UnknownTypeError, UnsupportTypeError, IsPreviewError)
 
 
-def parse_episodes(episodes_str, total):
+def parse_episodes(episodes_str: str, total: int):
     """ 将选集字符串转为列表 """
 
+    def reslove_negetive(value):
+        return value if value > 0 else value + total + 1
+
     # 解析字符串为列表
     print("全 {} 话".format(total))
-    if episodes_str == "all":
-        episode_list = list(range(1, total + 1))
-    elif re.match(r"(\d+(~\d+)?)(,\d+(~\d+)?)*", episodes_str):
+    if re.match(r"([\-\d\^\$]+(~[\-\d\^\$]+)?)(,[\-\d\^\$]+(~[\-\d\^\$]+)?)*", episodes_str):
+        episodes_str = episodes_str.replace("^", "1")
+        episodes_str = episodes_str.replace("$", "-1")
         episode_list = []
         for episode_item in episodes_str.split(","):
             if "~" in episode_item:
                 start, end = episode_item.split("~")
                 start, end = int(start), int(end)
-                assert end > start, "终点值（{}）应大于起点值（{}）".format(end, start)
+                start, end = reslove_negetive(start), reslove_negetive(end)
+                assert end >= start, "终点值（{}）应不小于起点值（{}）".format(end, start)
                 episode_list.extend(list(range(start, end + 1)))
             else:
-                episode_list.append(int(episode_item))
+                episode_item = int(episode_item)
+                episode_item = reslove_negetive(episode_item)
+                episode_list.append(episode_item)
     else:
         episode_list = []
 
@@ -80,21 +84,28 @@ def main():
         type=int,
         help="视频清晰度 120:4K, 116:1080P60, 112:1080P+, 80:1080P, 74:720P60, 64:720P, 32:480P, 16:360P",
     )
-    parser.add_argument("-n", "--num-threads", default=30, type=int, help="最大下载线程数")
-    parser.add_argument("-p", "--episodes", default="all", help="选集")
+    parser.add_argument("-n", "--num-threads", default=16, type=int, help="最大下载线程数")
+    parser.add_argument("-p", "--episodes", default="^~$", help="选集")
     parser.add_argument("-w", "--overwrite", action="store_true", help="强制覆盖已下载视频")
     parser.add_argument("-c", "--sess-data", default=None, help="输入 cookies")
     parser.add_argument("-y", "--yes", action="store_true", help="跳过下载询问")
+    parser.add_argument(
+        "--audio-quality", default=30280,
+        choices=[30280, 30232, 30216],
+        type=int,
+        help="音频码率等级 30280:320kbps, 30232:128kbps, 30216:64kbps",
+    )
     parser.add_argument(
         "--playlist-type", default="dpl", choices=["dpl", "m3u", "no"], help="播放列表类型，支持 dpl 和 m3u，输入 no 不生成播放列表",
     )
-    parser.add_argument("--path-type", default="rp", help="播放列表路径类型（rp：相对路径，ap：绝对路径）")
     parser.add_argument(
         "--danmaku", default="xml", choices=["xml", "ass", "no"], help="弹幕类型，支持 xml 和 ass，如果设置为 no 则不下载弹幕",
     )
     parser.add_argument(
         "--block-size", default=128, type=int, help="分块下载器的块大小，单位为 MB，默认为 128MB，设置为 0 时禁用分块下载",
     )
+    parser.add_argument("--abs-path", action="store_true", help="修改播放列表路径类型为绝对路径")
+    parser.add_argument("--use-mirrors", action="store_true", help="启用从多个镜像下载功能")
     parser.add_argument("--disable-proxy", action="store_true", help="禁用系统代理")
     parser.add_argument("--debug", action="store_true", help="debug 模式")
 
@@ -105,14 +116,15 @@ def main():
         "url": args.url,
         "dir": args.dir,
         "quality": args.quality,
+        "audio_quality": args.audio_quality,
         "episodes": args.episodes,
         "playlist_type": args.playlist_type,
-        "playlist_path_type": args.path_type.upper(),
+        "playlist_path_type": "AP" if args.abs_path else "RP",
         "overwrite": args.overwrite,
         "cookies": cookies,
         "type": args.type.lower(),
         "block_size": int(args.block_size * 1024 * 1024),
-    }
+    } >> AttrDict()
 
     # 匹配资源的 id 以及其对应所属类型
     # fmt: off
@@ -157,10 +169,10 @@ def main():
         sys.exit(1)
 
     if resource_id.avid or resource_id.bvid:
-        from bilili.video_parser.acg_video import get_title, get_list, get_playurl
+        from bilili.parser.acg_video import get_title, get_list, get_playurl
         bili_type = "acg_video"
     elif resource_id.season_id or resource_id.episode_id:
-        from bilili.video_parser.bangumi import get_title, get_list, get_playurl
+        from bilili.parser.bangumi import get_title, get_list, get_playurl
         bili_type = "bangumi"
 
     # 获取标题
@@ -173,9 +185,6 @@ def main():
     # 创建所需目录结构
     base_dir = touch_dir(os.path.join(config["dir"], repair_filename(title + " - bilibili")))
     video_dir = touch_dir(os.path.join(base_dir, "Videos"))
-    if args.overwrite:
-        shutil.rmtree(video_dir)
-        touch_dir(video_dir)
 
     # 获取需要的信息
     containers = [BililiContainer(video_dir=video_dir, type=args.type, **video) for video in get_list(resource_id)]
@@ -206,7 +215,7 @@ def main():
 
         # 解析视频 url
         try:
-            for playinfo in get_playurl(container, config["quality"]):
+            for playinfo in get_playurl(container, config["quality"], config["audio_quality"]):
                 container.append_media(
                     block_size=config["block_size"],
                     **playinfo
@@ -245,18 +254,18 @@ def main():
     if containers:
         # 状态检查与校正
         for i, container in enumerate(containers):
-            container_downloaded = os.path.exists(container.path)
+            container_downloaded = not container.check_needs_download(args.overwrite)
             symbol = "✓" if container_downloaded else "✖"
             if container_downloaded:
                 container._.merged = True
             print("{} {}".format(symbol, str(container)))
             for media in container.medias:
-                media_downloaded = os.path.exists(media.path) or container_downloaded
+                media_downloaded = not media.check_needs_download(args.overwrite) or container_downloaded
                 symbol = "✓" if media_downloaded else "✖"
                 if not container_downloaded:
                     print("    {} {}".format(symbol, media.name))
                 for block in media.blocks:
-                    block_downloaded = os.path.exists(block.path) or media_downloaded
+                    block_downloaded = not block.check_needs_download(args.overwrite) or media_downloaded
                     symbol = "✓" if block_downloaded else "✖"
                     block._.downloaded = block_downloaded
                     if not media_downloaded and args.debug:
@@ -279,8 +288,11 @@ def main():
         # 部署下载与合并任务
         merge_wait_flag = Flag(False)  # 合并线程池不能因为没有任务就结束
         # 因此要设定一个 flag，待最后合并结束后改变其值
-        merge_pool = ThreadPool(3, wait=merge_wait_flag)
-        download_pool = ThreadPool(args.num_threads)
+        merge_pool = ThreadPool(3, wait=merge_wait_flag, daemon=True)
+        download_pool = ThreadPool(args.num_threads, daemon=True, thread_globals_creator={
+            "thread_spider":spider.clone            # 为每个线程创建一个全新的 Session，因为 requests.Session 不是线程安全的
+                                                    # https://github.com/psf/requests/issues/1871
+        })
         for container in containers:
             merging_file = MergingFile(container.type, [media.path for media in container.medias], container.path,)
             for media in container.medias:
@@ -288,52 +300,48 @@ def main():
                 block_merging_file = MergingFile(None, [block.path for block in media.blocks], media.path)
                 for block in media.blocks:
 
-                    remote_file = RemoteFile(block.url, block.path, range=block.range)
+                    mirrors = block.mirrors if args.use_mirrors else []
+                    remote_file = RemoteFile(block.url, block.path, mirrors=mirrors, range=block.range)
 
-                    # 为下载挂载各种钩子，以修改状态
-                    @remote_file.on("before_download", middleware=block._)
-                    def before_download(file, middleware=None):
-                        middleware.downloading = True
+                    # 为下载挂载各种钩子，以修改状态，注意外部变量应当作为默认参数传入
+                    @remote_file.on("before_download")
+                    def before_download(file, status=block._):
+                        status.downloading = True
 
-                    @remote_file.on("updated", middleware=block._)
-                    def updated(file, middleware=None):
-                        middleware.size = file.size
+                    @remote_file.on("updated")
+                    def updated(file, status=block._):
+                        status.size = file.size
 
-                    @remote_file.on(
-                        "downloaded",
-                        middleware=block._,
-                        merging_file=merging_file,
-                        block_merging_file=block_merging_file,
-                    )
-                    def downloaded(file, middleware=None, merging_file=None, block_merging_file=None):
-                        middleware.downloaded = True
+                    @remote_file.on("downloaded")
+                    def downloaded(file, status=block._, merging_file=merging_file, block_merging_file=block_merging_file):
+                        status.downloaded = True
 
-                        if middleware.parent.downloaded:
+                        if status.parent.downloaded:
                             # 当前 media 的最后一个 block 所在线程进行合并（直接执行，不放线程池）
-                            middleware.downloaded = False
+                            status.downloaded = False
                             block_merging_file.merge()
-                            middleware.downloaded = True
+                            status.downloaded = True
 
                             # 如果该线程同时也是当前 container 的最后一个 block，就部署合并任务（放到线程池）
-                            if middleware.parent.parent.downloaded and not middleware.parent.parent.merged:
+                            if status.parent.parent.downloaded and not status.parent.parent.merged:
                                 # 为合并挂载各种钩子
-                                @merging_file.on("before_merge", middleware=middleware.parent.parent)
-                                def before_merge(file, middleware=None):
-                                    middleware.merging = True
+                                @merging_file.on("before_merge")
+                                def before_merge(file, status=status.parent.parent):
+                                    status.merging = True
 
-                                @merging_file.on("merged", middleware=middleware.parent.parent)
-                                def merged(file, middleware=None):
-                                    middleware.merging = False
-                                    middleware.merged = True
+                                @merging_file.on("merged")
+                                def merged(file, status=status.parent.parent):
+                                    status.merging = False
+                                    status.merged = True
 
                                 merge_pool.add_task(merging_file.merge, args=())
 
-                        middleware.downloading = False
+                        status.downloading = False
 
                     # 下载过的不应继续部署任务
                     if block._.downloaded:
                         continue
-                    download_pool.add_task(remote_file.download, args=(spider,))
+                    download_pool.add_task(remote_file.download, args=())
 
         # 启动线程池
         merge_pool.run()
@@ -362,9 +370,9 @@ def main():
         )
 
         # 准备监控
-        size, t = global_middleware.size, time.time()
+        size, t = global_status.size, time.time()
         while True:
-            now_size, now_t = global_middleware.size, time.time()
+            now_size, now_t = global_status.size, time.time()
             delta_size, delta_t = (
                 max(now_size - size, 0),
                 (now_t - t) if now_t - t > 1e-6 else 1e-6,
@@ -381,7 +389,7 @@ def main():
                     },
                     {
                         "left": "🌠 Downloading videos: "
-                    } if global_middleware.downloading else None,
+                    } if global_status.downloading else None,
                     [
                         {
                             "left": "{} ".format(str(container)),
@@ -390,37 +398,39 @@ def main():
                             ),
                         } if container._.downloading else None
                         for container in containers
-                    ] if global_middleware.downloading else None,
+                    ] if global_status.downloading else None,
                     {
-                        "left": global_middleware.size / global_middleware.total_size,
+                        "left": global_status.size / global_status.total_size,
                         "right": " {}/{} {}/s ⚡".format(
-                            size_format(global_middleware.size),
-                            size_format(global_middleware.total_size),
+                            size_format(global_status.size),
+                            size_format(global_status.total_size),
                             size_format(speed),
                         ),
-                    } if global_middleware.downloading else None,
+                    } if global_status.downloading else None,
                     {
                         "left": "🍰 Merging videos: "
-                    } if global_middleware.merging else None,
+                    } if global_status.merging else None,
                     [
                         {
                             "left": "{} ".format(str(container)),
                             "right": True
                         } if container._.merging else None
                         for container in containers
-                    ] if global_middleware.merging else None,
+                    ] if global_status.merging else None,
                     {
                         "left": sum([container._.merged for container in containers]) / len(containers),
                         "right": " {}/{} 🚀".format(
                             sum([container._.merged for container in containers]), len(containers),
                         ),
-                    } if global_middleware.merging else None,
+                    } if global_status.merging else None,
                 ]
             )
 
             # 检查是否已经全部完成
-            if global_middleware.downloaded and global_middleware.merged:
+            if global_status.downloaded and global_status.merged:
                 merge_wait_flag.value = True
+                download_pool.join()
+                merge_pool.join()
                 break
             try:
                 # 将刷新率稳定在 2fps
