@@ -1,8 +1,7 @@
 import re
-import json
 
-from bilili.tools import spider, regex_bangumi_ep
-from bilili.quality import gen_quality_sequence, quality_map
+from bilili.tools import spider
+from bilili.quality import gen_quality_sequence, video_quality_map, Media
 from bilili.utils.base import touch_url
 from bilili.api.exceptions import (
     ArgumentsError,
@@ -17,7 +16,10 @@ from bilili.api.exports import export_api
 @export_api(route="/get_season_id")
 def get_season_id(media_id: str) -> str:
     home_url = "https://www.bilibili.com/bangumi/media/md{media_id}".format(media_id=media_id)
-    season_id = re.search(r'"param":{"season_id":(\d+),"season_type":\d+}', spider.get(home_url).text).group(1)
+    season_id = ""
+    regex_season_id = re.compile(r'"param":{"season_id":(\d+),"season_type":\d+}')
+    if regex_season_id.search(spider.get(home_url).text):
+        season_id = regex_season_id.search(spider.get(home_url).text).group(1)
     return str(season_id)
 
 
@@ -28,16 +30,22 @@ def get_bangumi_title(media_id: str = "", season_id: str = "", episode_id: str =
     if media_id:
         home_url = "https://www.bilibili.com/bangumi/media/md{media_id}".format(media_id=media_id)
         res = spider.get(home_url)
-        title = re.search(r'<span class="media-info-title-t">(.*?)</span>', res.text).group(1)
+        regex_title = re.compile(r'<span class="media-info-title-t">(.*?)</span>')
+        if regex_title.search(res.text):
+            title = regex_title.search(res.text).group(1)
+        else:
+            title = "呐，我也不知道是什么标题呢～"
     elif season_id or episode_id:
         if season_id:
             play_url = "https://www.bilibili.com/bangumi/play/ss{season_id}".format(season_id=season_id)
         else:
             play_url = "https://www.bilibili.com/bangumi/play/ep{episode_id}".format(episode_id=episode_id)
         res = spider.get(play_url)
-        title = re.search(
-            r'<a href=".+" target="_blank" title="(.*?)" class="media-title">(?P<title>.*?)</a>', res.text
-        ).group("title")
+        regex_title = re.compile(r'<a href=".+" target="_blank" title="(.*?)" class="media-title">(?P<title>.*?)</a>')
+        if regex_title.search(res.text):
+            title = regex_title.search(res.text).group("title")
+        else:
+            title = "呐，我也不知道是什么标题呢～"
     return title
 
 
@@ -67,9 +75,16 @@ def get_bangumi_list(episode_id: str = "", season_id: str = ""):
 
 @export_api(route="/bangumi/playurl")
 def get_bangumi_playurl(
-    avid: str = "", bvid: str = "", episode_id: str = "", cid: str = "", quality: int = 120, type: str = "dash"
+    avid: str = "",
+    bvid: str = "",
+    episode_id: str = "",
+    cid: str = "",
+    quality: int = 120,
+    audio_quality: int = 30280,
+    type: str = "dash",
 ):
-    quality_sequence = gen_quality_sequence(quality)
+    video_quality_sequence = gen_quality_sequence(quality, type=Media.VIDEO)
+    audio_quality_sequence = gen_quality_sequence(audio_quality, type=Media.AUDIO)
     play_api = "https://api.bilibili.com/pgc/player/web/playurl?avid={avid}&bvid={bvid}&ep_id={episode_id}&cid={cid}&qn={quality}"
     if type == "flv":
         touch_message = spider.get(
@@ -81,7 +96,7 @@ def get_bangumi_playurl(
             raise IsPreviewError()
 
         accept_quality = touch_message["result"]["accept_quality"]
-        for quality in quality_sequence:
+        for quality in video_quality_sequence:
             if quality in accept_quality:
                 break
 
@@ -92,9 +107,10 @@ def get_bangumi_playurl(
             {
                 "id": i + 1,
                 "url": segment["url"],
+                "mirrors": segment["backup_url"],
                 "quality": quality,
-                "height": quality_map[quality]["height"],
-                "width": quality_map[quality]["width"],
+                "height": video_quality_map[quality]["height"],
+                "width": video_quality_map[quality]["width"],
                 "size": segment["size"],
                 "type": "flv_segment",
             }
@@ -104,49 +120,66 @@ def get_bangumi_playurl(
         result = []
         play_api_dash = play_api + "&fnver=0&fnval=16&fourk=1"
         play_info = spider.get(
-            play_api_dash.format(avid=avid, bvid=bvid, episode_id=episode_id, cid=cid, quality=quality_sequence[0])
+            play_api_dash.format(
+                avid=avid, bvid=bvid, episode_id=episode_id, cid=cid, quality=video_quality_sequence[0]
+            )
         ).json()
+
         if play_info["code"] != 0:
             raise CannotDownloadError(play_info["code"], play_info["message"])
         if play_info["result"].get("dash") is None:
             raise UnsupportTypeError("dash")
-        if play_info["code"] != 0:
-            raise CannotDownloadError(play_info["code"], play_info["message"])
         if play_info["result"]["is_preview"] == 1:
             raise IsPreviewError()
 
-        accept_quality = set([video["id"] for video in play_info["result"]["dash"]["video"]])
-        for quality in quality_sequence:
-            if quality in accept_quality:
+        accept_video_quality = set([video["id"] for video in play_info["result"]["dash"]["video"]])
+        for video_quality in video_quality_sequence:
+            if video_quality in accept_video_quality:
                 break
+        else:
+            video_quality = 120
 
-        for video in play_info["result"]["dash"]["video"]:
-            if video["id"] == quality:
-                result.append(
-                    {
-                        "id": 1,
-                        "url": video["base_url"],
-                        "quality": quality,
-                        "height": video["height"],
-                        "width": video["width"],
-                        "size": touch_url(video["base_url"], spider)[0],
-                        "type": "dash_video",
-                    }
-                )
+        accept_audio_quality = set([audio["id"] for audio in play_info["result"]["dash"]["audio"]])
+        for audio_quality in audio_quality_sequence:
+            if audio_quality in accept_audio_quality:
                 break
-        for audio in play_info["result"]["dash"]["audio"]:
-            result.append(
-                {
-                    "id": 2,
-                    "url": audio["base_url"],
-                    "quality": quality,
-                    "height": None,
-                    "width": None,
-                    "size": touch_url(audio["base_url"], spider)[0],
-                    "type": "dash_audio",
-                }
-            )
-            break
+        else:
+            audio_quality = 30280
+
+        if play_info["result"]["dash"]["video"]:
+            videos = play_info["result"]["dash"]["video"]
+            for video in videos:
+                if video["id"] == video_quality:
+                    result.append(
+                        {
+                            "id": 1,
+                            "url": video["base_url"],
+                            "mirrors": video["backup_url"],
+                            "quality": quality,
+                            "height": video["height"],
+                            "width": video["width"],
+                            "size": touch_url(video["base_url"], spider)[0],
+                            "type": "dash_video",
+                        }
+                    )
+                    break
+        if play_info["result"]["dash"]["audio"]:
+            audios = play_info["result"]["dash"]["audio"]
+            for audio in audios:
+                if audio["id"] == audio_quality:
+                    result.append(
+                        {
+                            "id": 2,
+                            "url": audio["base_url"],
+                            "mirrors": audio["backup_url"],
+                            "quality": audio_quality,
+                            "height": None,
+                            "width": None,
+                            "size": touch_url(audio["base_url"], spider)[0],
+                            "type": "dash_audio",
+                        }
+                    )
+                    break
         return result
     elif type == "mp4":
         raise UnsupportTypeError("mp4")
