@@ -30,7 +30,9 @@ def test_get_playurl(type):
     cid = "144541892"
     episode_id = "300998"
     try:
-        play_list = get_bangumi_playurl(avid=avid, bvid=bvid, cid=cid, episode_id=episode_id, quality=120, type=type)
+        play_list = get_bangumi_playurl(
+            avid=avid, bvid=bvid, cid=cid, episode_id=episode_id, quality=120, audio_quality=30280, type=type
+        )
     # 可能 GitHub Action 由于地区限制无法获取
     except CannotDownloadError:
         pass
